/* gcc shield_dEdx.c -o shield_dEdx -ldedx -Wall */
/* TODO: allow to override I-value. */

#include<dedx.h>
#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<time.h>
#include "shield_dEdx.h"

#define STRSIZE 127

int main(int argc, char *argv[]) {

  FILE *fp;
  int vmajor,vminor,vpatch,vsvn;
  int err = 0;
  int i,j, prog_select = 0, target_select = 0;
  dedx_workspace *ws;
  /* one extra for He-4 MSTAR in case of ICRU for E > 250MeV/u */
  dedx_config *cfg[SHIELD_IONS+1];

  char filename[STRSIZE];
  char progname[STRSIZE];
  char buffer[STRSIZE];
  float stp;
  time_t curtime;
  struct tm *loctime;
  const int *mat_list;

  char *user = getenv("USER");

  if (argc != 3) {
    printf("Usage:\n");
    printf("%s program ICRU_ID\n", argv[0]);
    printf("Use -1 for either program or ICRU_ID to see list of availble materials.\n\n");
    printf("(C) 2012 by Niels Bassler <bassler@phys.au.dk>.\n");
    printf("This is part of SHIELD-HITxxA\n");
    exit(-1);
  }

  prog_select = atoi(argv[1]);
  target_select = atoi(argv[2]);

  if (prog_select > 3) {
    printf("No such program: %i\n", prog_select);
    prog_select = -1;  /* print list and exit */
  }

  if ((target_select > 278) && (target_select != 906)) {
    printf("No such material: %i\n",target_select);
    target_select = -1; /* print list and exit */
  }

  if (prog_select < 1) {
    printf("Available programs:\n");
    printf("  1 PSTAR + MSTAR\n");
    printf("  2 ICRU \n");
    printf("  3 Bethe-Bloch with Linhard-Scharff.\n");
    exit(-1);
  }


  if (prog_select == 1) {
    sprintf(progname,"PSTAR + MSTAR");
    if (target_select < 1)
      mat_list = dedx_get_material_list(DEDX_MSTAR);
  }
  if (prog_select == 2) {
    sprintf(progname,"ICRU");
    if (target_select < 1)
      mat_list = dedx_get_material_list(DEDX_ICRU);
  }
  if (prog_select == 3) {
    sprintf(progname,"BETHE_EXT00");
    if (target_select < 1)
      mat_list = dedx_get_material_list(DEDX_BETHE_EXT00);
  }

  if (target_select < 1) {
    i = 0;
    while (mat_list[i] != -1) {
      printf("%i: %s\n", mat_list[i], dedx_get_material_name(mat_list[i]));
      i++;
    }
    exit(-1);
  }

  ws = dedx_allocate_workspace(SHIELD_IONS+1, &err);
  if (err != 0)
    printf("Bummer during init: %i\n",err);

  for(i = 0; i <= SHIELD_IONS; i++) {
    cfg[i] = calloc(1, sizeof(dedx_config));
  }

  /* set targets and ions for all configs */
  for(i = 0; i <= SHIELD_IONS; i++) {
    cfg[i]->target = target_select;
    cfg[i]->ion = i + 1;
  }
  cfg[SHIELD_IONS]->ion = DEDX_HELIUM; /* last placeholder for MSTAR Helium */
  cfg[SHIELD_IONS]->program = DEDX_MSTAR;
  dedx_load_config(ws, cfg[SHIELD_IONS], &err);

  /* PSTAR and MSTAR */
  if (prog_select == 1) {
    cfg[0]->program = DEDX_PSTAR;
    dedx_load_config(ws, cfg[0], &err);
    if (err != 0) {
      printf("Bummer when loading ion %i with prog %i in material %i: %i\n"
	     ,DEDX_PROTON,cfg[0]->program,cfg[0]->target,err);
      if (err == 202)
	printf("Material combination not supported.\n");
      exit(-1);
    }
    for ( i = 1; i < SHIELD_IONS; i++) {
      cfg[i]->program = DEDX_MSTAR;
      dedx_load_config(ws, cfg[i], &err);
      if (err != 0) {
	printf("Bummer when loading ion %i with prog %i in material %i: %i\n"
	       ,cfg[i]->ion,cfg[i]->program,cfg[i]->target,err);
	if (err == 202)
	  printf("Material combination not supported.\n");
	exit(-1);
      }
    }
  }

  /* ICRU 49 + 73 */
  if (prog_select == 2) {
    if (err != 0) {
      printf("Bummer when loading ion %i with prog %i in material %i: %i\n",
	     cfg[SHIELD_IONS]->ion,
	     cfg[SHIELD_IONS]->program,
	     cfg[SHIELD_IONS]->target,err);
      if (err == 202)
	printf("Material combination not supported.\n");
      exit(-1);
    }
    for(i = 0; i < SHIELD_IONS; i++)  /* do not include last element, it was set before */
      cfg[i]->program = DEDX_ICRU;
  }

  if (prog_select == 3)
    for(i = 0; i < SHIELD_IONS; i++)  /* do not include last element, it was set before */
      cfg[i]->program = DEDX_BETHE_EXT00;

  /* populate all tables exept for PSTAR/MSTAR which is already done */
  if (prog_select > 1) {
    for (i=0; i < SHIELD_IONS; i++) {
      dedx_load_config(ws, cfg[i], &err);
      if (err != 0) {
	printf("Bummer xxx when loading ion %i with prog %i in material %i: %i\n"
	       ,cfg[i]->ion,cfg[i]->program,cfg[i]->target,err);
	if (err == 202)
	  printf("Material combination not supported.\n");
	exit(-1);
      }
    }
  }


  printf("Preparing stopping power table for %s using %s.\n",
	 dedx_get_material_name(target_select), progname);

  /* now open file and build table. Header: */
  sprintf(filename, "%s.txt", shield_filename_table[target_select]);
  fp = fopen(filename,"w");

  dedx_get_version(&vmajor,&vminor,&vpatch,&vsvn);
  fprintf(fp,"* Stopping power tables calculated with libdEdx, version %i.%i.%i-svn%i\n",
	  vmajor,vminor,vpatch,vsvn);
  gethostname(buffer,STRSIZE),
  fprintf(fp,"* by user %s on host %s.\n",user,buffer);
  curtime = time (NULL);
  loctime = localtime (&curtime);
  strftime(buffer, STRSIZE, "%c\n",loctime);
  fprintf(fp,"* Date of this calculation: %s",buffer);
  strftime(buffer, STRSIZE, "%Y%m%d_%H%M%S",loctime);
  fprintf(fp,"* Timestamp: %s\n",buffer);
  fprintf(fp,"* Target material id: %i\n", target_select);
  fprintf(fp,"* Target material name: %s\n", dedx_get_material_name(target_select));

  if (prog_select == 1)
    sprintf(buffer,"DEDX_PSTAR for z = 1 and DEDX_MSTAR for z > 1");
  if (prog_select == 2)
    sprintf(buffer,"DEDX_ICRU49 + DEDX_ICRU73 + DEDX_MSTAR for He-4 with E > 250 MeV");
  if (prog_select == 3)
    sprintf(buffer,"DEDX_BETHE_EXT00 using mean excitation potential I = %.2f eV",
	    cfg[0]->i_value);

  fprintf(fp,"* Algorithm: %s\n", buffer);
  fprintf(fp,"* Energy [MeV/nucleon] dE/dx [MeV cm2/g] for ions 1<=z<=%i \n"
	  ,SHIELD_IONS);

  /* build data table */
  for (i = 0; i < SHIELD_ETABLE; i++) {
    fprintf(fp, "%6.3E ",shield_energy_table[i]);
    for (j = 0; j < SHIELD_IONS; j++) {
      stp = dedx_get_stp(ws, cfg[j], shield_energy_table[i],&err);
      if (err != 0) {
	printf("Warning: error code %i while getting stopping powers:", err);
	printf(" ion %i at %6.3e MeV/nucleon.\n",
	       cfg[j]->ion,shield_energy_table[i]);
	stp = 0;
      }
      if (cfg[j]->program == DEDX_ICRU && cfg[j]->ion == DEDX_HELIUM && shield_energy_table[i] > 250.0) {
	printf("   Falling back to MSTAR in case of He-4 with E> 250 MeV/nucleon\n");
	stp = dedx_get_stp(ws, cfg[SHIELD_IONS],
			   shield_energy_table[i],&err);
      }
      fprintf(fp, "%6.3E ",stp);
    } /* end ion list */
    fprintf(fp, "\n");
  } /* end table list */

  fclose(fp);
  dedx_free_workspace(ws,&err);
  for (i=0;i<=SHIELD_IONS;i++)
    free(cfg[i]);

  printf("Wrote %s.\n",filename);

  return 0;
}
